@@ -7,7 +7,7 @@
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -61,9 +61,12 @@ int _gnutls_dh_set_group (gnutls_session_t session, bigint_t gen,
                           bigint_t prime);
 
 static inline int
-_gnutls_dh_get_allowed_prime_bits (gnutls_session_t session)
+_gnutls_dh_get_min_prime_bits (gnutls_session_t session)
 {
-  return session->internals.dh_prime_bits;
+  if (session->internals.priorities.dh_prime_bits != 0)
+    return session->internals.priorities.dh_prime_bits;
+  else
+    return gnutls_sec_param_to_pk_bits(GNUTLS_PK_DH, session->internals.priorities.level);
 }
 
 void _gnutls_handshake_internal_state_clear (gnutls_session_t);
