@@ -918,7 +918,7 @@ gnutls_pkcs12_generate_mac (gnutls_pkcs12_t pkcs12, const char *pass)
 
   /* Generate the key.
    */
-  result = _gnutls_pkcs12_string_to_key (3 /*MAC*/, salt, sizeof (salt),
+  result = _gnutls_pkcs12_string_to_key (GNUTLS_MAC_SHA1, 3 /*MAC*/, salt, sizeof (salt),
                                          iter, pass, sizeof (key), key);
   if (result < 0)
     {
@@ -1001,7 +1001,8 @@ cleanup:
 int
 gnutls_pkcs12_verify_mac (gnutls_pkcs12_t pkcs12, const char *pass)
 {
-  uint8_t key[20];
+  uint8_t key[MAX_HASH_SIZE];
+  char oid[MAX_OID_SIZE];
   int result;
   unsigned int iter;
   int len;
@@ -1009,8 +1010,11 @@ gnutls_pkcs12_verify_mac (gnutls_pkcs12_t pkcs12, const char *pass)
   gnutls_datum_t tmp = { NULL, 0 }, salt =
   {
   NULL, 0};
-  uint8_t sha_mac[20];
-  uint8_t sha_mac_orig[20];
+  uint8_t mac_output[MAX_HASH_SIZE];
+  uint8_t mac_output_orig[MAX_HASH_SIZE];
+  gnutls_mac_algorithm_t algo;
+  unsigned mac_len, key_len;
+  
 
   if (pkcs12 == NULL)
     {
@@ -1028,22 +1032,40 @@ gnutls_pkcs12_verify_mac (gnutls_pkcs12_t pkcs12, const char *pass)
       iter = 1;                 /* the default */
     }
 
+  len = sizeof(oid);
+  result =
+    asn1_read_value(pkcs12->pkcs12, "macData.mac.digestAlgorithm.algorithm",
+                    oid, &len);
+  if (result != ASN1_SUCCESS)
+    {
+      gnutls_assert();
+      return _gnutls_asn2err(result);
+    }
+
+  algo = _gnutls_x509_oid_to_mac(oid);
+  if (algo == GNUTLS_MAC_UNKNOWN)
+    {
+      gnutls_assert();
+      return GNUTLS_E_UNKNOWN_HASH_ALGORITHM;
+    }
+
+  mac_len = _gnutls_hmac_get_algo_len(algo);
+  key_len = mac_len;
 
   /* Read the salt from the structure.
    */
   result =
     _gnutls_x509_read_value (pkcs12->pkcs12, "macData.macSalt", &salt);
-  if (result != ASN1_SUCCESS)
+  if (result < 0)
     {
       gnutls_assert ();
-      result = _gnutls_asn2err (result);
       goto cleanup;
     }
 
   /* Generate the key.
    */
-  result = _gnutls_pkcs12_string_to_key (3 /*MAC*/, salt.data, salt.size,
-                                         iter, pass, sizeof (key), key);
+  result = _gnutls_pkcs12_string_to_key (algo, 3 /*MAC*/, salt.data, salt.size,
+                                         iter, pass, key_len, key);
   if (result < 0)
     {
       gnutls_assert ();
@@ -1063,7 +1085,7 @@ gnutls_pkcs12_verify_mac (gnutls_pkcs12_t pkcs12, const char *pass)
 
   /* MAC the data
    */
-  result = _gnutls_hmac_init (&td1, GNUTLS_MAC_SHA1, key, sizeof (key));
+  result = _gnutls_hmac_init (&td1, algo, key, key_len);
   if (result < 0)
     {
       gnutls_assert ();
@@ -1073,11 +1095,11 @@ gnutls_pkcs12_verify_mac (gnutls_pkcs12_t pkcs12, const char *pass)
   _gnutls_hmac (&td1, tmp.data, tmp.size);
   _gnutls_free_datum (&tmp);
 
-  _gnutls_hmac_deinit (&td1, sha_mac);
+  _gnutls_hmac_deinit (&td1, mac_output);
 
-  len = sizeof (sha_mac_orig);
+  len = sizeof (mac_output_orig);
   result =
-    asn1_read_value (pkcs12->pkcs12, "macData.mac.digest", sha_mac_orig,
+    asn1_read_value (pkcs12->pkcs12, "macData.mac.digest", mac_output_orig,
                      &len);
   if (result != ASN1_SUCCESS)
     {
@@ -1086,7 +1108,8 @@ gnutls_pkcs12_verify_mac (gnutls_pkcs12_t pkcs12, const char *pass)
       goto cleanup;
     }
 
-  if (memcmp (sha_mac_orig, sha_mac, sizeof (sha_mac)) != 0)
+  if ((unsigned)len != mac_len ||
+      memcmp (mac_output_orig, mac_output, len) != 0)
     {
       gnutls_assert ();
       return GNUTLS_E_MAC_VERIFY_FAILED;
