@@ -34,13 +34,14 @@
 #include <string.h>
 #include <unistd.h>
 #include <errno.h>
+#include <common.h>
 #include "udp-serv.h"
 #include "list.h"
 
 typedef struct {
   gnutls_session_t session;
   int fd;
-  struct sockaddr * cli_addr;
+  struct sockaddr_storage *cli_addr;
   socklen_t cli_addr_size;
 } priv_data_st;
 
@@ -53,7 +54,7 @@ static ssize_t pull_func(gnutls_transport_ptr_t p, void * data, size_t size);
 void udp_server(const char* name, int port, int mtu)
 {
     int sock, ret;
-    struct sockaddr_in cli_addr;
+    struct sockaddr_storage cli_addr;
     socklen_t cli_addr_size;
     char buffer[MAX_BUFFER];
     priv_data_st priv;
@@ -88,7 +89,7 @@ void udp_server(const char* name, int port, int mtu)
         if (ret > 0)
           {
             memset(&prestate, 0, sizeof(prestate));
-            ret = gnutls_dtls_cookie_verify(&cookie_key, &cli_addr, sizeof(cli_addr), buffer, ret, &prestate);
+            ret = gnutls_dtls_cookie_verify(&cookie_key, &cli_addr, cli_addr_size, buffer, ret, &prestate);
             if (ret < 0) /* cookie not valid */
               {
                 priv_data_st s;
@@ -96,11 +97,11 @@ void udp_server(const char* name, int port, int mtu)
                 memset(&s,0,sizeof(s));
                 s.fd = sock;
                 s.cli_addr = (void*)&cli_addr;
-                s.cli_addr_size = sizeof(cli_addr);
+                s.cli_addr_size = cli_addr_size;
                 
                 printf("Sending hello verify request to %s\n", human_addr ((struct sockaddr *)
-                  &cli_addr, sizeof(cli_addr), buffer, sizeof(buffer)));
-                gnutls_dtls_cookie_send(&cookie_key, &cli_addr, sizeof(cli_addr), &prestate, (gnutls_transport_ptr_t)&s, push_func);
+                  &cli_addr, cli_addr_size, buffer, sizeof(buffer)));
+                gnutls_dtls_cookie_send(&cookie_key, &cli_addr, cli_addr_size, &prestate, (gnutls_transport_ptr_t)&s, push_func);
 
                 /* discard peeked data*/
                 recvfrom(sock, buffer, sizeof(buffer), 0, (struct sockaddr*)&cli_addr, &cli_addr_size);
@@ -108,7 +109,7 @@ void udp_server(const char* name, int port, int mtu)
               }
             printf ("Accepted connection from %s\n",
                             human_addr ((struct sockaddr *)
-                                        &cli_addr, sizeof(cli_addr), buffer,
+                                        &cli_addr, cli_addr_size, buffer,
                                         sizeof (buffer)));
           }
         else
@@ -120,8 +121,8 @@ void udp_server(const char* name, int port, int mtu)
 
         priv.session = session;
         priv.fd = sock;
-        priv.cli_addr = (struct sockaddr *)&cli_addr;
-        priv.cli_addr_size = sizeof(cli_addr);
+        priv.cli_addr = &cli_addr;
+        priv.cli_addr_size = cli_addr_size;
 
         gnutls_transport_set_ptr (session, &priv);
         gnutls_transport_set_push_function (session, push_func);
@@ -143,10 +144,26 @@ void udp_server(const char* name, int port, int mtu)
 
         for(;;)
           {
-            do {
-              ret = gnutls_record_recv_seq(session, buffer, MAX_BUFFER, sequence);
-            } while(ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
+            do 
+              {
+                ret = gnutls_record_recv_seq(session, buffer, MAX_BUFFER, sequence);
+                if (ret == GNUTLS_E_HEARTBEAT_PING_RECEIVED)
+                  gnutls_heartbeat_pong(session, 0);
+              }
+            while (ret == GNUTLS_E_INTERRUPTED || ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_HEARTBEAT_PING_RECEIVED);
 
+            if (ret == GNUTLS_E_REHANDSHAKE)
+              {
+                fprintf (stderr, "*** Received hello message\n");
+                do
+                  {
+                    ret = gnutls_handshake (session);
+                  }
+                while (ret == GNUTLS_E_INTERRUPTED ||
+                       ret == GNUTLS_E_AGAIN);
+                
+                if (ret == 0) continue;
+              }
             if (ret < 0)
               {
                 fprintf(stderr, "Error in recv(): %s\n", gnutls_strerror(ret));
@@ -157,16 +174,20 @@ void udp_server(const char* name, int port, int mtu)
                 printf("EOF\n\n");
                 break;
               }
+              
             buffer[ret] = 0;
             printf("received[%.2x%.2x%.2x%.2x%.2x%.2x%.2x%.2x]: %s\n", sequence[0], sequence[1], sequence[2],
-              sequence[3], sequence[4], sequence[5], sequence[6], sequence[7], buffer);
+                   sequence[3], sequence[4], sequence[5], sequence[6], sequence[7], buffer);
 
-            /* reply back */
-            ret = gnutls_record_send(session, buffer, ret);
-            if (ret < 0)
+            if (check_command(session, buffer) == 0)
               {
-                fprintf(stderr, "Error in send(): %s\n", gnutls_strerror(ret));
-                break;
+                /* reply back */
+                ret = gnutls_record_send(session, buffer, ret);
+                if (ret < 0)
+                  {
+                    fprintf(stderr, "Error in send(): %s\n", gnutls_strerror(ret));
+                    break;
+                  }
               }
           }
       }
@@ -190,7 +211,13 @@ char c;
   
   tv.tv_sec = 0;
   tv.tv_usec = ms * 1000;
-  
+
+  while(tv.tv_usec >= 1000000)
+    {
+      tv.tv_usec -= 1000000;
+      tv.tv_sec++;
+    }
+
   ret = select(priv->fd+1, &rfds, NULL, NULL, &tv);
 
   if (ret <= 0)
@@ -214,7 +241,7 @@ static ssize_t push_func (gnutls_transport_ptr_t p, const void * data, size_t si
 {
 priv_data_st *priv = p;
 
-  return sendto(priv->fd, data, size, 0, priv->cli_addr, priv->cli_addr_size);
+  return sendto(priv->fd, data, size, 0, (struct sockaddr*)priv->cli_addr, priv->cli_addr_size);
 }
 
 static ssize_t pull_func(gnutls_transport_ptr_t p, void * data, size_t size)
